

set -ex



test -L $PREFIX/lib/libnvToolsExt.so
test -f $PREFIX/lib/pkgconfig/nvToolsExt*.pc
test -f $PREFIX/targets/sbsa-linux/include/nvToolsExtCuda.h
test -f $PREFIX/targets/sbsa-linux/include/nvtx3/nvToolsExtCuda.h
test -L $PREFIX/targets/sbsa-linux/lib/libnvToolsExt.so
exit 0
